-- Function: audit()

-- DROP FUNCTION audit();

CREATE OR REPLACE FUNCTION audit()
  RETURNS SETOF jp_audit AS
$BODY$
declare	Psys_config_set		sys_config.set%type;
       	java_control_used	sys_config.set%type;
       	stationery	        sys_config.set%type;
       	stat_type	        sys_config.set%type;
       	Pnparams_current	nparams.current_period%type;
       	w_period		dparams.period%type;
       	Pparams_current		pparams.period%type;
        Pnparams_creditors	nparams.creditors%type;
       	msg1			char(30);
       	msg2			char(30);
       	msg3			char(30);
        v1			decimal(16,2);
       	v2			decimal(16,2);
       	v3			decimal(16,2);
       	v4			decimal(16,2);
       	w1			decimal(16,2);
       	a1			decimal(16,2);
       	b1			decimal(16,2);
       	c1			decimal(16,2);
       	w_java			decimal(16,2);
       	w_java_inv		decimal(16,2);
       	w_java_cr 		decimal(16,2);
       	w_vatsum 		decimal(16,2);
       	false			integer;
       	w_tmp			decimal(16,2);
       	tot_cost		decimal(16,2);
       	wk_cost			decimal(16,2);
       	v_cost			decimal(16,2);
       	unalloc_tot		decimal(16,2);
       	ledger_tot		decimal(16,2);
       	cr_notes		decimal(16,2);
       	invs    		decimal(16,2);
       	obal			decimal(16,2);
        cbal			decimal(16,2);
        trans			decimal(16,2);
	trans_tot		decimal(16,2);
       	Pnullcnt		integer;
       	diff			decimal(16,2);
        tot_depn		decimal(16,2);
        w_amount        decimal(16,2);
        w_os            decimal(16,2);

       	Pnlbals_cod		nlbals.cod%type;
       	Pnlbals_cc		nlbals.cc%type;
       	Pnlbals_period		nlbals.period%type;
       	Pnlbals_bop_bal		nlbals.bop_bal%type;
       	Pnlbals_cur_bal		nlbals.cur_bal%type;
       	Pnlbals_ytd_bal		nlbals.ytd_bal%type;

        ihead_row		ihead%rowtype;
        sanal_row		sanal%rowtype;
        nlbatch_row		nlbatch%rowtype;
        cust_row		cust%rowtype;
        supplier_row		supplier%rowtype;
        pledger_row		pledger%rowtype;
        asset_reg_row	        asset_reg%rowtype;
        nominal_row	        nominal%rowtype;
        chdetail_row		chdetail%rowtype;

        Pproduct_cost		product.cost%type;
	    results			jp_audit; 
        w_ver           jp_verify; 
        w_rows			integer;  
        last_checkpoint record; 
        w_today         date;  
        sales_period    record;
        creditors_period record; 
        found_checkpoint boolean;
        some_wild_date  date;
        debtors_nl_balance decimal(16,2);    
        creditors_nl_balance decimal(16,2);
        err_code             integer;
        err_text             char(60);   
        all_depots           integer;
        cust_check             record;
        alloc_row              record; 
        run_mode               char(6); 
        analsl_row             jp_audit;


begin

    false := 0;
    all_depots := 0;  
    run_mode := 'detail';
    some_wild_date := now(); --'03/11/2053';
    
    perform * from nrbld_custaged_at(some_wild_date,0,0);
    perform * from nrbld_suppaged_at(some_wild_date,0,0,'FULL',0);

    java_control_used := 1;

    select current_period, creditors
    into Pnparams_current, Pnparams_creditors from nparams;

    select period into w_period from dparams;

    select period into Pparams_current from pparams;

-- 1.	Check Aged Debt Analysis Against Debtors in Nominal

     select sum(balance) into v1 from cust_age
     where period = jp_period(some_wild_date)
      and bad_debt = 'N';

    select sum(amount)
      into debtors_nl_balance
      from nltrans a, nparams b, reserved_ac c
     where (a.period >= b.boy_period -- All periods from beginning of year (inc CP)
            or
            a.tmp = 'Y') and     --     Unposted batches
            a.cod = c.account_code and
            c.key_name = 'debtors';

    select sum(amount)
    into w_java
    from sledger
    where batch is null
      and cod is not null
      and source not in ('BR', 'BD');

    if (not FOUND) or (w_java is null) then
        w_java := 0.00;
    end if;

    v2 := debtors_nl_balance + w_java;

    msg1 := 'Aged Debt Analysis';

    msg2 := 'Nominal Debtors + JAVA TX';

    msg1 := 'Aged Debt Analysis';

    if w_java <> 0 then
       msg2 := 'Nominal Debtors + JAVA TX';
    else
       msg2 := 'Nominal Debtors';
    end if;

    results.seq := 1;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 2. Check Aged Debt Analysis Against Sales Control Balance

    select * from sl_open_bal(some_wild_date, all_depots) 
        into err_code, err_text, v2;

    select sum(amount)
      into trans_tot
       from sledger
      where period >= some_wild_date
        and cod is not null;

    if trans_tot is null then 
	trans_tot := 0.00;
    end if;	

    v2 := v2 + trans_tot;



    if v2 is null then
	    v2 := 0.00;
    end if;
    
    msg2 := 'Sales Ledger Control + JAVA TX';
    
    results.seq := 2;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
       
    return next results;

-- 2a. Check Nominal Debtors Against Sales Control

    
    msg1 := 'Nominal Debtors + JAVA TX';
    msg2 := 'Sales Ledger Control + JAVA TX';
    
--    v2 already has sales control figure

    v1 = debtors_nl_balance + w_java;

    
    results.seq := 2;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;
   

-- 3.  Check Invoice Archive Headers against Sales Control

   --- Defunct

-- 4.  Check Invoice Archive Headers against Ledger Transactions

    select sum(tot)
    into v1
    from ihead
    where ihead.period = w_period
      and (ihead.doc_type = 'IN' or
           ihead.doc_type = 'CA');

    if (not FOUND) OR (v1 is null) then
        v1 := 0.00;
    end if;

    select sum(amount)
    into v2
    from sledger
    where sledger.typ = 2
      and sledger.period = w_period
      and sledger.cod is not null;

    if (not FOUND) or (v2 is null) then
	    v2 := 0.00;
    end if;
    
    msg1 := 'Invoices On Archive';
    msg2 := 'Ledger Transactions';

    results.seq := 4;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 5.  Check Credit Note Archive Headers against Sales Control

  -- Defunct
  
-- 6.  Check Credit Note Archive Headers against Ledger Transactions

    select sum(tot)
    into v1
    from ihead
    where ihead.period = w_period
      and ihead.doc_type = 'CR';

    if v1 is null then
        v1 := 0.00;
    end if;


    select sum(amount)
    into v2
    from sledger
    where sledger.typ = 12
      and sledger.period = w_period
      and sledger.cod is not null;

    if v2 is null then
	    v2 := 0.00;
    end if;

    msg2 = 'Ledger Transactions';

    results.seq := 6;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := 0 - v2;
    results.diff := (v1 - (0 - v2));
    return next results;

-- 7.  Check Hire Details on Archive against Hire Sales Analysis

   -- Defunct

-- 8.  Check Consumable Details on Archive against Consumable Sales Analysis

    -- Defunct  

-- 9. Check Cost of Consumable sales against sanal.qty sold * product.cost

    -- Defunct

-- 10.  Check Disposal Details on Archive against Disposal Sales Analysis

    -- Defunct
   

-- 11.  Check Disposal Details on Archive against Disposal Table

     select sum(b.goods)
      into v1
    from ihead a, iddetail b
    where a.doc_type = b.doc_type
      and a.location = b.location
      and a.ref = b.ref
      and a.cust = b.cust
      and a.period = w_period;

    if (not FOUND) or (v1 is null) then
	    v1 := 0.00;
    end if;
  
    select sum(qty * unit_sell)
    into a1
    from disposals
    where disposals.sales_period = w_period
    and qty > 0;
     
    if (not FOUND) or (a1 is null) then
	    a1 := 0.00;
    end if;  
                                                                  
    select sum(qty * unit_sell)
    into b1
    from disposals
    where disposals.sales_period = w_period
    and qty < 0;

    if (not FOUND) or (b1 is null) then
	    b1 := 0.00;
    end if;
    
    v2 := a1 - b1;

    msg2 := 'Disposals List';
    results.seq := 11;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 12.  Check for unbalanced batches

    msg1 := 'Unbalanced Batches';
    msg2 := null;
    v1 := null;

    for nlbatch_row in
        select *
        from nlbatch
        where stat <> 'I'		-- only those not yet updated
    loop

	    select sum(amount)
	    into w1
	    from nltrans
	    where nltrans.source = nlbatch_row.source
	      and nltrans.batch  = nlbatch_row.seq;

	    if w1 is null then
	        w1 := 0.00;
	    end if;

	    if w1 <> 0.00 then
	       msg2 := nlbatch_row.location || '/' || nlbatch_row.source || nlbatch_row.seq;
	       v2   := w1;
           results.seq := 12;
           results.msg1 := msg1;
           results.v1 := v1;
           results.msg2 := msg2;
           results.v2 := v1;
           results.diff := v2;
           return next results;
	    end if;

    end loop;

-- 13 / 14 / 15. Check for customer accounts out of balance.

    --
    -- Accounts are only out of balance if the sum of their amounts
    -- <> the sum of their os
    --

    msg1 := 'Unbalanced Debtor';
    msg3 := 'Unallocated Diff ';
    msg2 := null;
    v1 := null;

    for cust_check in  
        select depot, cod, nam
        from cust
       where bad_debt = 'N'
     
    loop
    
        select sum(amount) as amount,
               sum(CASE WHEN (typ > 10 )
                            THEN (os * '-1.00')
                        ELSE os
                   END) as os
          into w_amount, w_os
          from sledger
          where depot = cust_check.depot
            and cod = cust_check.cod;

          if w_amount <> w_os then
              msg1 := 'Unbalanced Debtor ';
              msg2 := cust_check.depot || '/' || cust_check.cod;
             
              results.seq := 13;
              results.msg1 := msg1;
              results.v1 := w_amount;
              results.msg2 := msg2;
              results.v2 := w_os;
              results.diff := w_amount - w_os;
              return next results;

              if run_mode = 'detail' then

                  --
                  --  Since we have a difference analyse the allocations
                  --

                  for analsl_row in select * from analyse_slalloc(cust_check.depot, 
                                                 cust_check.cod)
                  loop
                      
                      return next analsl_row;

                  end loop;  

              end if;

          end if;

    end loop;


-- 16. Check Sales VAT Analysis

--- Redundant


--		P U R C H A S E   L E D G E R


-- 17.  Check The Supplier Master Total Against the Balance On Nominal

    select sum(balance)
    into v1
    from supplier_age
    where period = jp_period(some_wild_date);

    select sum(amount)
      into creditors_nl_balance
      from nltrans a, nparams b
     where (a.period >= b.boy_period -- All periods from beginning of year (inc CP)
            or
            a.period is null) 
            and        --     Unposted batches
            a.cod in ( 
		select cod from nominal 
		where parent in ( 
			select account_code from reserved_ac
			where reserved_ac.key_name = 'creditors')
			);

   v2 = creditors_nl_balance;

   v2 = 0 - v2;


   msg1 := 'Supplier Aged List';
   msg2 := 'Creditors Nominal + JAVA TX';
  

    results.seq := 17;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 18.  Check Supplier List Against Control Account Total

    -- Sum of supplier.balance is in v1 from last query

    select sum(amount)
    into v2
    from pledger
    where pledger.supplier is null
      and pledger.period = Pparams_current;

    v2 := v2 + w_java;

    if w_java <> 0 then
       msg1 := 'Supplier Aged List ';
       msg2 := 'Control Account Bal + JAVA TX';
    else
       msg1 := 'Supplier Aged List ';
       msg2 := 'Control Account Balance ';
    end if;

    if java_control_used = false then

        results.seq := 18;
        results.msg1 := msg1;
        results.v1 := v1;
        results.msg2 := msg2;
        results.v2 := v2;
        results.diff := v1 - v2;
        return next results;
    end if;

-- 19. Check Purchase VAT Analysis.

    msg1 := 'Purchase VAT Analysis';
    msg2 := 'Purchase Inv/Cr';

    w_vatsum := 0.00;

    select sum(pdetails.goods + pdetails.vat)
    into cr_notes
    from pdetails, pledger
    where pdetails.trans_ser = pledger.ser
      and pledger.period = Pparams_current
      and typ = 11;

    if cr_notes is null then
       cr_notes := 0.00;
    end if;

    cr_notes := 0.00 - cr_notes;

    select sum(pdetails.goods + pdetails.vat)
    into invs
    from pdetails, pledger
    where pdetails.trans_ser = pledger.ser
      and pledger.period = Pparams_current
      and typ = 2;

    if invs is null then
       invs := 0.00;
    end if;

    w_vatsum := cr_notes + invs;

    v1 := 0.00;

    select amount
    into v1
    from pledger
    where pledger.supplier is NULL
      and pledger.typ = 2
      and pledger.period = Pparams_current;

    v2 := 0.00;

    select amount
    into v2
    from pledger
    where pledger.supplier is NULL
      and pledger.typ = 11
      and pledger.period = Pparams_current;

    v1 := v1 + v2;			-- v2 will be negative

    v1 := v1 + w_java;

    if w_java <> 0 then
       msg1 := 'Purchase VAT Analysis';
       msg2 := 'Purchase Inv/Cr + JAVA TX';
    else
       msg1 := 'Purchase VAT Analysis';
       msg2 := 'Purchase Inv/Cr';
    end if;
    if java_control_used = false then

        results.seq := 19;
        results.msg1 := msg1;
        results.v1 := w_vatsum;
        results.msg2 := msg2;
        results.v2 := v1;
        results.diff := (w_vatsum - v1);
        return next results;
    end if;

-- 20. Check Supplier Accounts Against Their Transactions

---    msg1 := 'Unbalanced Creditor';

---    for supplier_row in
---    select *
---   from supplier
---    loop

---    select sum(os)
---	   into v2
--- from pledger
---	where pledger.supplier = supplier_row.cod;

---	if v2 is null then
---	    v2 := 0.00;
---	end if;

---	if v2 <> supplier_row.balance then  
--  Force a fix ------------------------------------------------------  
--        update supplier  
--           set balance = v2
--           where cod = supplier_row.cod; 
----------------------------------------------------------------------
---	    msg2 := supplier_row.cod || ' - ' || substring(supplier_row.nam for 20);
---	    v1 := supplier_row.balance;

---            results.seq := 20;
---            results.msg1 := msg1;
---            results.v1 := v1;
---            results.msg2 := msg2;
---            results.v2 := v2;
---           results.diff := v1 - v2;
---            return next results;

---	end if;

---    end loop;

-- 21.  Check Purchase Control Vs Transactions

    if java_control_used = false then
        for pledger_row in
        select *
        from pledger
        where pledger.supplier is null
          and pledger.period = Pparams_current
          and pledger.typ <> 1
        loop

	    msg1 := get_ptrans(pledger_row.typ);
	    msg2 := 'Purchase Ledger Control';

	    select sum(amount) into v2
	    from pledger
	    where pledger.typ = pledger_row.typ and
	      pledger.period = Pparams_current and
	      pledger.supplier is not null;

	    if v2 is null then
	        v2 := 0.00;
	    end if;

            if pledger_row.typ = 11 then
                v2 := v2 + w_java_cr;
                msg2 := 'Purchase Ledger Control + JAVA TX';
            end if;

            if pledger_row.typ = 2 then
                v2 := v2 + w_java_inv;
                msg2 := 'Purchase Ledger Control + JAVA TX';
            end if;

	    v1 := pledger_row.amount;

            results.seq := 21;
            results.msg1 := msg1;
            results.v1 := v1;
            results.msg2 := msg2;
            results.v2 := v2;
            results.diff := v1 - v2;
            return next results;

        end loop;
    end if;

--		N O M I N A L   L E D G E R

-- 22. Check that 'BALANC' account is zero.

    msg1 := '"BALANC" Account';
    msg2 := '';

    select cur_bal
    into v1
    from nlbals
    where nlbals.cod = 'BALANC'
      and nlbals.period = Pnparams_current;

    v2 := 0.00;

    results.seq := 22;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 23. Check Current Trial Balance

    msg1 := 'Current Trial Balance';
    msg2 := ''; 
     
    create temp table audit_currnltb as
    select S1.cod, sum(amount) as amount,pm.period,s1.description 
    from nominal s1, nominal s2, nltrans t, period_map pm, nparams ctl
    where s1.lft NOT BETween s2.lft and s2.rgt
      and s2.cod = 'TRADIN'
      and t.period BETWEEN pm.boy_period and pm.period
      and pm.period = ctl.current_period
      and t.cod = s1.cod
      and tmp = 'N'
    group by 1,3,4

    union ALL

    select account_code, sum(amount) as amount, pm.period, 'Accumulated Profit'
    from nominal s1, nominal s2, nltrans t, reserved_ac r, period_map pm, nparams ctl
    where s1.lft between s2.lft and s2.rgt
    and s2.cod = 'TRADIN' 
    and t.period >= pm.boy_period
    and t.period < pm.period 
    and pm.period = ctl.current_period
    and t.cod = s1.cod
    and tmp = 'N' 
    and r.key_name = 'capital'
    group by 1,3,4 

    union all

    select S1.cod, sum(amount) as amount, t.period, s1.description
    from nominal s1, nominal s2, nltrans t, nparams ctl
    where s1.lft BETween s2.lft and s2.rgt
      and s2.cod = 'TRADIN' 
      and t.period = ctl.current_period
      and t.cod = s1.cod 
      and tmp = 'N' 
    group by 1,3,4 
    order by 3,1; 


    select sum(amount)
    into v1
    from audit_currnltb ; 
    
    drop table audit_currnltb;

    v2 := 0.00;

    results.seq := 23;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 24. Check Year to Date Trial Balance

    msg1 := 'Year To Date Trial Balance';
    msg2 := ''; 
    
    select sum(amount)
      into v1
      from nominal s1, nominal s2, nltrans t, period_map pm, nparams ctl
     where s1.lft between s2.lft and s2.rgt 
       and s2.cod = 'BALANC'
       and t.period between pm.boy_period and pm.period
       and pm.period = ctl.current_period
       and t.cod = s1.cod
       and tmp = 'N';
     

    v2 := 0.00;

    results.seq := 24;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

-- 25. Check Leaf Node Accounts Vs Transactions

    
    
   
-- 26.  Check The Sum Of Nominal Transactions = Zero

    msg1 := 'Posted Double Entry Integrity';
    msg2 := '';

    select sum(amount)
    into v1
    from nltrans
    where nltrans.period = Pnparams_current
      and nltrans.tmp = 'N';

    results.seq := 26;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := 0.00;
    results.diff := v1 - 0.00;
    return next results;

-- 27.  Unposted Double Entry Integrity

    msg1 := 'Un-p Double Entry Integrity';
    msg2 := '';

    select sum(amount)
    into v1
    from nltrans
    where nltrans.period = Pnparams_current
      and nltrans.tmp = 'Y';

    if v1 is null then
        v1 := 0.00;
    end if;

    results.seq := 27;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := 0.00;
    results.diff := v1 - 0.00;
    return next results;

-- 28.  Check invoice headers against the sum of their detail lines.

    for ihead_row in
    select *
    from ihead
    where period = w_period
    loop

        select sum(goods + vat)
        into a1
        from ihdetail
        where ihdetail.location = ihead_row.location
          and ihdetail.ref      = ihead_row.ref
          and ihdetail.doc_type = ihead_row.doc_type;

        if a1 is null then
            a1 := 0.0;
        end if;

        select sum(goods + vat)
        into c1
        from iodetail
        where iodetail.location = ihead_row.location
          and iodetail.ref      = ihead_row.ref
          and iodetail.doc_type = ihead_row.doc_type;

        if c1 is null then
            c1 := 0.00;
        end if;

        select sum(goods + vat)
        into b1
        from iddetail
        where iddetail.location = ihead_row.location
          and iddetail.ref      = ihead_row.ref
          and iddetail.doc_type = ihead_row.doc_type;

        if b1 is null then
            b1 := 0.00;
        end if;

        w1 := a1 + b1 + c1;

        if w1 <> ihead_row.tot then
           diff = w1 - ihead_row.tot;
           if diff < 0 then
              diff = 0 - diff;
           end if;
           if diff <> .01 then
               msg1 = 'Header: ' || ihead_row.location|| '/'||
	                  ihead_row.cust || '/' || ihead_row.ref;
               msg2 = 'Detail Lines';

               results.seq := 28;
               results.msg1 := msg1;
               results.v1 := ihead_row.tot;
               results.msg2 := msg2;
               results.v2 := w1;
               results.diff := ihead_row.tot - w1;
               return next results;
           end if;
        end if;
    end loop;

--		P L A N T   R E G I S T E R

-- 29.  Check Plant Cost Vs Nominal

    for asset_reg_row in
    select *
    from asset_reg
    loop

        select sum(pcost.qty * pcost.unit_cost),
	       sum(pcost.qty * pcost.unit_depn)
        into v1, tot_depn		-- may as well get depn at this pt
        from pcost			-- will need it for next check
        where pcost.asset_reg = asset_reg_row.cod;

	select cur_bal
	into v2
	from nlbals
	where nlbals.period = Pnparams_current
	  and nlbals.cod = asset_reg_row.nominal_cost;

	if (v2 is null) or (not FOUND) then
	    v2 := 0.00;
	end if;

	select sum(amount)
	into w_tmp
	from nltrans
	where nltrans.cod = asset_reg_row.nominal_cost
	  and nltrans.tmp = 'Y';

	if w_tmp is null then
	    w_tmp := 0.00;
	end if;

	v2 := v2 + w_tmp;

        msg2 := 'Nominal Equipment At Cost';
	msg1 := asset_reg_row.description;

        results.seq := 29;
        results.msg1 := msg1;
        results.v1 := v1;
        results.msg2 := msg2;
        results.v2 := v2;
        results.diff := v1 - v2;
        return next results;

	v1 := tot_depn;

	select cur_bal
	into v2
	from nlbals
	where nlbals.period = Pnparams_current
	  and nlbals.cod = asset_reg_row.nominal_accum_depn;

	if (v2 is null) or (not FOUND) then
	    v2 := 0.00;
	end if;

	select sum(amount)
	into w_tmp
	from nltrans
	where nltrans.cod = asset_reg_row.nominal_accum_depn
	  and nltrans.tmp = 'Y';

	if w_tmp is null then
	    w_tmp := 0.00;
	end if;

	v2 := v2 + w_tmp;

	v2 := 0 - v2;

	msg2 := 'Nominal Accumulated Depn';

        results.seq := 30;
        results.msg1 := msg1;
        results.v1 := v1;
        results.msg2 := msg2;
        results.v2 := v2;
        results.diff := v1 - v2;
        return next results;

    end loop;

    select sum(pcost.qty * pcost.unit_cost) -
               sum(pcost.qty * pcost.unit_depn)
    into v1
    from pcost
    where pcost.asset_reg = 'H';

    v2 := total_register();

    msg1 := 'Sum of Pcost';
    msg2 := 'Plant Register Report';

    results.seq := 31;
    results.msg1 := msg1;
    results.v1 := v1;
    results.msg2 := msg2;
    results.v2 := v2;
    results.diff := v1 - v2;
    return next results;

    select count(*)
    into Pnullcnt
    from nlbatch
    where seq is null
    or source is null;

    if Pnullcnt > 0 then
	if Pnullcnt = 1 then
	    msg1 := Pnullcnt || ' null batch is present.';
	else
	    msg1 := Pnullcnt || ' null batches are present.';
	end if;

        results.seq := 32;
        results.msg1 := msg1;
        results.v1 := null;
        results.msg2 := null;
        results.v2 := null;
        results.diff := null;
        return next results;

    end if;

    for nlbatch_row in
    select *
    from nlbatch
    where nlbatch.stat = 'I'
    loop

        msg1 := ' Incomplete batch :- ' || nlbatch_row.source || ' ' || nlbatch_row.seq;

        results.seq := 33;
        results.msg1 := msg1;
        results.v1 := null;
        results.msg2 := null;
        results.v2 := null;
        results.diff := null;
        return next results;

    end loop;

    for chdetail_row in
       select *
       from chdetail
       where date_from is null or date_started is null
    loop

        msg1 := ' Null date on contract :- ' || chdetail_row.contract;

        results.seq := 34;
        results.msg1 := msg1;
        results.v1 := null;
        results.msg2 := null;
        results.v2 := null;
        results.diff := null;
        return next results;

    end loop; 
  
    --
    -- Verify all Debtors Batches Against the Nominal
    -- 
    
    found_checkpoint := false;
    
    for last_checkpoint in 
        select period
        from nltrans, reserved_ac
        where nltrans.cod = reserved_ac.account_code 
          and reserved_ac.key_name = 'debtors'  
          and nltrans.source = 'CP'
        order by period desc
    loop     
        found_checkpoint := true;
        exit;
    
    end loop;
    
   select current_date 
     into w_today;
     
    if found_checkpoint then
    
        for sales_period in
            select distinct period  
            from sledger  
            where period between last_checkpoint.period and w_today
            order by period
         loop 
         
             for w_ver in
                 select *
                   from verify_dr_nl(sales_period.period,1) 
             loop
    
                 results.seq := 32;
                 results.v1 := w_ver.subledger_total;
                 results.msg2 := sales_period.period || ' ' ||w_ver.source 
                                 || '/' || w_ver.batch || ' ' ||w_ver.direction;
                 results.msg1 := w_ver.comment;
                 results.v2 := w_ver.nominal_total;
                 results.diff := v1 - v2;
                 return next results;
             
              end loop;
         
         end loop; 
         
         for creditors_period in
            select distinct period  
            from pledger  
            where period between last_checkpoint.period and w_today
            order by period
         loop 
         
             for w_ver in
                 select *
                   from verify_cr_nl(creditors_period.period,1) 
             loop
    
                 results.seq := 33;
                 results.v1 := w_ver.subledger_total;
                 results.msg2 := creditors_period.period || ' ' || w_ver.source 
                               || '/' || w_ver.batch || ' ' ||w_ver.direction;
                 results.msg1 := w_ver.comment;
                 results.v2 := w_ver.nominal_total;
                 results.diff := v1 - v2;
                 return next results;
             
              end loop;
         
         end loop;
         
                
     end if;




  
   
    
    return;
   

end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION audit() OWNER TO postgres;
